#****************************************************************************
#
#  Module: 1_AddToAzureTenant.ps1
#
#  Purpose: Setup secRMM Property Variables in Azure.
#
#  Copyright (c) 2023 Squadra Technologies 
#   
#****************************************************************************

. $PSScriptRoot\_TenantSpecifics.ps1;
. $PSScriptRoot\_secRMMSpecifics_DO_NOT_CHANGE.ps1;

Add-Type -AssemblyName Microsoft.VisualBasic;

# az login;
try { 
 $var = Get-AzureADTenantDetail;
} 
catch [Microsoft.Open.Azure.AD.CommonLibrary.AadNeedAuthenticationException] { 
	Connect-AzureAD;
}

$TenantId = $(az account show --query tenantId);

$NameOfAzFunctionApp = $TenantId + $NameOfAzFunctionApp;

$Subscriptions = $(az account list --query "[].{name:name}" --output tsv);
$SubscriptionName = $null;
switch ($Subscriptions.Count)
{
    0 { Write-Error "No Azure Subscriptions exist in tenant $TenantId, script cannot proceed...";}
    1 { $SubscriptionName = $Subscriptions[0]; }
	Default {
		$SubscriptionName = $Subscriptions | Out-GridView -OutputMode Single -Title "Please select an Azure Subscription";
    }
}

if ($SubscriptionName -ne $null) {
	az account set --subscription $SubscriptionName;
}

$SubscriptionId = az account show --query id --output tsv;

$Groups0 = $(az group list --query "[].{name:name}" --output tsv);
$Groups0String = $Groups0 -join ',';
$Groups0String = "Create New...," + $Groups0String;
$Groups = $Groups0String -split ',';
$NameOfAzResourceGroupForStorageAccount = $Groups | Out-GridView -OutputMode Single -Title "Please select an Azure Resource Group for the Azure Storage Account";

if ($NameOfAzResourceGroupForStorageAccount -eq 'Create New...') {
	$NameOfAzResourceGroupForStorageAccount = [Microsoft.VisualBasic.Interaction]::InputBox('Enter a new name for the Azure Resource Group for the Azure Storage Account', 'secRMM Azure Functions Setup', '');
}

$AzGroupExists = $(az group exists --name $NameOfAzResourceGroupForStorageAccount --resource-group $NameOfAzResourceGroupForStorageAccount);
if ($AzGroupExists -eq $false) {
	az group create --name $NameOfAzResourceGroupForStorageAccount --location $LocationOfAzResourceGroup --output json;
}

$CreateAzStorageAccount = $false;

$StorageAccounts0 = $(az storage account list --resource-group $NameOfAzResourceGroupForStorageAccount --query "[].{name:name}" --output tsv);
$StorageAccounts0String = $StorageAccounts0 -join ',';
$StorageAccounts0String = "Create New...," + $StorageAccounts0String;
$StorageAccounts = $StorageAccounts0String -split ',';
$NameOfStorageAccount = $StorageAccounts | Out-GridView -OutputMode Single -Title "Please select an Azure Storage Account";

if ($NameOfStorageAccount -eq 'Create New...') {
	$NameOfStorageAccount = [Microsoft.VisualBasic.Interaction]::InputBox('Enter a new name for the Azure Storage Account', 'secRMM Azure Functions Setup', '');
}

# Check if storage account exists - Method 1
# $AzStorageAccountExists1 = $(az storage account list --resource-group $NameOfAzResourceGroupForStorageAccount --query [].name --output table);
# if ($AzStorageAccountExists1 -eq "") {
#	$CreateAzStorageAccount = $true;
# }

# Check if storage account exists - Method 2
$AzStorageAccountExists2 = $(az storage account check-name --name $NameOfStorageAccount) | ConvertFrom-Json;
$CreateAzStorageAccount = $AzStorageAccountExists2.nameAvailable;

if ($CreateAzStorageAccount -eq $true) {
	# Create a storage app for the storage
	
	$LocationOfAzResourceGroups = $(az account list-locations --query "[].{name:name}" --output tsv);
	$LocationOfAzResourceGroup = $LocationOfAzResourceGroups | Out-GridView -OutputMode Single -Title "Please select a location for the new Azure Storage Account";

	$StorageSkus = @('Premium_LRS', 'Premium_ZRS', 'Standard_GRS', 'Standard_GZRS', 'Standard_LRS', 'Standard_RAGRS', 'Standard_RAGZRS', 'Standard_ZRS');
	$StorageSku = $StorageSkus | Out-GridView -OutputMode Single -Title "Please select a storage Sku for the new Azure Storage Account";

	az storage account create --name $NameOfStorageAccount --resource-group $NameOfAzResourceGroupForStorageAccount --location $LocationOfAzResourceGroup --sku $StorageSku --output json;
}

$FunctionApp = (func azure functionapp list-functions $NameOfAzFunctionApp);

$AzFuncAppServicePrincipalId = $null;

if ($FunctionApp -eq $null) {

	$LocationOfAzResourceGroups2 = $(az account list-locations --query "[].{name:name}" --output tsv);
	$LocationOfAzResourceGroup2 = $LocationOfAzResourceGroups2 | Out-GridView -OutputMode Single -Title "Please select a location for the new Azure Function";

	# Create the FunctionApp within the AppService using a consumption plan
	az functionapp create `
		--name $NameOfAzFunctionApp `
		--resource-group $NameOfAzResourceGroupForStorageAccount `
		--storage-account $NameOfStorageAccount `
		--functions-version '4' `
		--os-type 'Windows' `
		--runtime 'powershell' `
		--runtime-version '7.2' `
		--consumption-plan-location $LocationOfAzResourceGroup2 `
		--output json `
		--https-only true;
		# --consumption-plan-location <REGION>
		# --disable-app-insights true

	az functionapp identity assign --name $NameOfAzFunctionApp --resource-group $NameOfAzResourceGroupForStorageAccount;

	$AzFuncAppServicePrincipalId = $(az ad sp list --display-name $NameOfAzFunctionApp --query [].id --output tsv);

	if ($AzFuncAppServicePrincipalId -eq $null) {
		Write-Host "waiting";
		Do {
			Write-Host "... " -NoNewline;
			Start-Sleep -s 5
			$AzFuncAppServicePrincipalId = $(az ad sp list --display-name $NameOfAzFunctionApp --query [].id --output tsv);
		}
		while ($AzFuncAppServicePrincipalId -eq $null)
		Write-Host "done.";
	}
}

if ($AzFuncAppServicePrincipalId -eq $null) {
	$AzFuncAppServicePrincipalId = $(az ad sp list --display-name $NameOfAzFunctionApp --query [].id --output tsv);

	if ($AzFuncAppServicePrincipalId -eq $null) {
		Write-Host "waiting";
		Do {
			Write-Host "... " -NoNewline;
			Start-Sleep -s 5
			$AzFuncAppServicePrincipalId = $(az ad sp list --display-name $NameOfAzFunctionApp --query [].id --output tsv);
		}
		while ($AzFuncAppServicePrincipalId -eq $null)
		Write-Host "done.";
	}
}

push-location $PSScriptRoot;
# Publish the Azure Function App to Azure Tenant
func azure functionapp publish $NameOfAzFunctionApp;
#func azure functionapp publish $NameOfAzFunctionApp --nozip;
pop-location;

# The app ID of the API to assign permissions, Microsoft Graph: 00000003-0000-0000-c000-000000000000
$appId = "00000003-0000-0000-c000-000000000000";

$permissionsToAdd = @();

# API permissions required by secRMM
# if ($NameOfAzFunctionApp.Contains($NameOfAzFunctionApp[0])) { # Az:Group:?
#	 $permissionsToAdd = "User.Read.All", "Group.Read.All", "GroupMember.Read.All";
# }
# elseif ($NameOfAzFunctionApp.Contains($NameOfAzFunctionApp[1])) { # Az:User:?
#	$ permissionsToAdd = "User.Read.All";
# }
# elseif ($NameOfAzFunctionApp.Contains($NameOfAzFunctionApp[2])) { # Az:Computer:?
#	 $permissionsToAdd = "Device.Read.All";
# }

$permissionsToAdd = "Group.Read.All", "GroupMember.Read.All", "User.Read.All", "Device.Read.All";

if ($permissionsToAdd.Count -ne 0) {

	$app = Get-AzureADServicePrincipal -Filter "AppId eq '$appId'";

	foreach ($permission in $permissionsToAdd)
	{
	   $role = $app.AppRoles | where Value -Like $permission | Select-Object -First 1
	   New-AzureADServiceAppRoleAssignment -Id $role.Id -ObjectId $AzFuncAppServicePrincipalId -PrincipalId $AzFuncAppServicePrincipalId -ResourceId $app.ObjectId;
	}
}
