#****************************************************************************
#
#  Module: run.ps1 for secRMM Az:User:?
#
#  Purpose: Azure Function for secRMM Property Variables in Azure.
#
#  Copyright (c) 2023 Squadra Technologies 
#   
#****************************************************************************

# User.Read.All

using namespace System.Net

# Input bindings are passed in via param block.
param($Request, $TriggerMetadata)

#=============================================================================
function Convert-AzureSidToObjectId {
  param([String] $azureSid)
  $byteArray = [System.Byte[]]::CreateInstance([System.Byte], 16);
  $place = 0;
  $blocks = $azureSid.Replace("S-1-12-1-", "").Split("-", [System.StringSplitOptions]::RemoveEmptyEntries);
  foreach ($i in $blocks) {
    [Buffer]::BlockCopy([bitconverter]::GetBytes([int64]::Parse($i)), 0, $byteArray, $place, 4);
    $place += 4;
  }
  $Guid = [Guid]::new($byteArray);
  return($Guid);
}

#=============================================================================

# Write to the Azure Functions log stream.
Write-Host "PowerShell HTTP trigger function is processing a secRMM request for a user property.";

# Interact with query parameters or the body of the request.
$AzUserProperty = $Request.Query.AzUserProperty;
$AzUserSid = $Request.Query.UserSid;

Write-Host "AzUserProperty: $AzUserProperty";
Write-Host "AzUserSid: $AzUserSid";

$body = "";

if (($AzUserProperty) -and
	($AzUserSid)) {

	$ObjectId =	(Convert-AzureSidToObjectId -azureSid $AzUserSid).toString();

	if ($ObjectId) {
		Write-Host "ObjectId: $ObjectId";

		$msiEndpoint = $env:MSI_ENDPOINT;
		$msiSecret = $env:MSI_SECRET;

		Write-Host "msiEndpoint: $msiEndpoint";
		# Write-Host "msiSecret: $msiSecret";

		$apiVersion = "2017-09-01";
		$resourceURI = "https://graph.microsoft.com";
		$tokenAuthURI = $msiEndpoint + "?resource=$resourceURI&api-version=$apiVersion";
		$tokenResponse = Invoke-RestMethod -Method Get -Headers @{"Secret"="$msiSecret"} -Uri $tokenAuthURI;
		$accessToken = $tokenResponse.access_token;

		$graphApiVersion = "v1.0";
		$graphApiHeader = @{
				'Content-Type'='application/json'
				'Authorization'="Bearer " + $accessToken
				# 'ConsistencyLevel'='eventual'
				};

		# https://graph.microsoft.com/v1.0/users/67a3c4f4-0934-417d-87b4-b2ab26e9bdcd?$select=surname
		$graphURI = 'https://graph.microsoft.com/' + $graphApiVersion + '/users/' + $ObjectId + '?$select=' + $AzUserProperty;
		Write-Host "graphURI(1): $graphURI";

		$AzUserPropertyValue = (Invoke-RestMethod -Uri $graphURI -Headers $graphApiHeader -Method Get).$AzUserProperty; # wierd, if you put .Value, it does not work
		Write-Host "graphURI(1) returned from call...";

		if ($AzUserPropertyValue) {
			Write-Host "AzUserPropertyValue: $AzUserPropertyValue";

			$body = $AzUserPropertyValue;
		}
	}
}

Write-Host "Returning body: $body";

# Associate values to output bindings by calling 'Push-OutputBinding'.
Push-OutputBinding -Name Response -Value ([HttpResponseContext]@{
    StatusCode = [HttpStatusCode]::OK
    Body = $body
})
