#****************************************************************************
#
#  Module: run.ps1 for secRMM Az:Computer:?
#
#  Purpose: Azure Function for secRMM Property Variables in Azure.
#
#  Copyright (c) 2023 Squadra Technologies 
#   
#****************************************************************************

# Device.Read.All

using namespace System.Net

# Input bindings are passed in via param block.
param($Request, $TriggerMetadata)

# Write to the Azure Functions log stream.
Write-Host "PowerShell HTTP trigger function is processing a secRMM request for a computer property."

# Interact with query parameters or the body of the request.
$AzComputerProperty = $Request.Query.AzComputerProperty;
$AzComputerName = $Request.Query.ComputerName;

Write-Host "AzComputerProperty: $AzComputerProperty";
Write-Host "AzComputerName: $AzComputerName";

$body = "";

if (($AzComputerProperty) -and
	($AzComputerName)) {

	$msiEndpoint = $env:MSI_ENDPOINT;
	$msiSecret = $env:MSI_SECRET;

	Write-Host "msiEndpoint: $msiEndpoint";
	# Write-Host "msiSecret: $msiSecret";

	$apiVersion = "2017-09-01";
	$resourceURI = "https://graph.microsoft.com";
	$tokenAuthURI = $msiEndpoint + "?resource=$resourceURI&api-version=$apiVersion";
	$tokenResponse = Invoke-RestMethod -Method Get -Headers @{"Secret"="$msiSecret"} -Uri $tokenAuthURI;
	$accessToken = $tokenResponse.access_token;

	$graphApiVersion = "v1.0";
	$graphApiHeader = @{
            'Content-Type'='application/json'
            'Authorization'="Bearer " + $accessToken
		    # 'ConsistencyLevel'='eventual'
            };

	# https://graph.microsoft.com/v1.0/devices?$filter=startswith(displayName,'test1')&$select=operatingsystem
	$graphURI = ("https://graph.microsoft.com/v1.0/devices?`$filter=startswith(displayName,'{0}')&$select={1}" -f $AzComputerName, $AzComputerProperty); 
	Write-Host "graphURI(1): $graphURI";

	$AzComputerPropertyValue = (Invoke-RestMethod -Uri $graphURI -Headers $graphApiHeader -Method Get).Value.$AzComputerProperty;
	Write-Host "graphURI(1) returned from call...";

	if ($AzComputerPropertyValue) {
		Write-Host "AzComputerPropertyValue: $AzComputerPropertyValue";
		
		$body = $AzComputerPropertyValue;
	}
}

Write-Host "Returning body: $body";

# Associate values to output bindings by calling 'Push-OutputBinding'.
Push-OutputBinding -Name Response -Value ([HttpResponseContext]@{
    StatusCode = [HttpStatusCode]::OK
    Body = $body
})
